@@ -1,7 +1,7 @@
 /*
   curs_port.c
 
-  Copyright 2009, 2010, 2011, 2014, 2016, 2019 Free Software Foundation, Inc.
+  Copyright 2009, 2010, 2011, 2014, 2016, 2019, 2022 Free Software Foundation, Inc.
 
   This file is part of GNU Guile-Ncurses.
 
@@ -168,17 +168,38 @@ gucu_newterm (SCM type, SCM outp, SCM inp)
   SCM outp_fileno = scm_fileno (outp);
   int c_outp_fileno_orig = scm_to_int (outp_fileno);
   int c_outp_fileno = dup (c_outp_fileno_orig);
-  FILE *c_outp = fdopen (c_outp_fileno, "wb+");
-  if (c_outp == NULL)
+  FILE *c_outp;
+
+  if (c_outp_fileno == -1)
     return scm_from_int (2);
+  c_outp = fdopen (c_outp_fileno, "wb+");
+  if (c_outp == NULL)
+    {
+      close (c_outp_fileno);
+      return scm_from_int (2);
+    }
 
   SCM inp_fileno = scm_fileno (inp);
   int c_inp_fileno_orig = scm_to_int (inp_fileno);
   int c_inp_fileno = dup (c_inp_fileno_orig);
-  FILE *c_inp = fdopen (c_inp_fileno, "rb");
+  FILE *c_inp;
+
+  if (c_inp_fileno == -1)
+    {
+      fclose (c_outp);
+      close (c_outp_fileno);
+      return scm_from_int (1);
+    }
+
+  c_inp = fdopen (c_inp_fileno, "rb");
 
   if (c_inp == NULL)
-    return scm_from_int (1);
+    {
+      close (c_inp_fileno);
+      fclose (c_outp);
+      close (c_outp_fileno);
+      return scm_from_int (1);
+    }
 
   /* N.B.: Since we've duplicated these file descriptors, we need
      to close the ports on which the originals were based, so that
@@ -205,8 +226,10 @@ gucu_getwin (SCM port)
 {
   FILE *fp;
   WINDOW *c_win;
+#ifndef GUCU_USE_COOKIE
   SCM s_c;
   unsigned char c;
+#endif
 
   SCM_ASSERT (scm_is_true (scm_input_port_p (port)), port, SCM_ARG1,
               "getwin");
@@ -267,8 +290,6 @@ gucu_putwin (SCM win, SCM port)
 
 #ifdef GUCU_USE_COOKIE
   {
-    char *debug_str;
-    size_t debug_len;
     SCM out_string_port;
     size_t i;
 
