@@ -1,7 +1,7 @@
 /*
   form_func.c
 
-  Copyright 2009, 2010, 2014, 2016 Free Software Foundation, Inc.
+  Copyright 2009, 2010, 2014, 2016, 2019 Free Software Foundation, Inc.
 
   This file is part of GNU Guile-Ncurses.
 
@@ -36,63 +36,62 @@
 #error "No curses.h file included"
 #endif
 
-#include "compat.h"
 #include "form_func.h"
 #include "form_type.h"
 #include "type.h"
 
-extern scm_t_bits form_tag;
-extern scm_t_bits field_tag;
+extern SCM form_fo_type;
+extern SCM field_fo_type;
 
 /* Errors */
 
 void
 form_bad_state_error (const char *funcname)
 {
-  scm_misc_error (funcname, "bad state", SCM_BOOL_F);
+  scm_misc_error (funcname, "bad state", SCM_EOL);
 }
 
 void
 form_connected_error (const char *funcname)
 {
-  scm_misc_error (funcname, "field is already connected to a form", SCM_BOOL_F);
+  scm_misc_error (funcname, "field is already connected to a form", SCM_EOL);
 }
 
 void
 form_current_field_error (const char *funcname)
 {
-  scm_misc_error (funcname, "the field is the current field", SCM_BOOL_F);
+  scm_misc_error (funcname, "the field is the current field", SCM_EOL);
 }
 
 void
 form_invalid_field_error (const char *funcname)
 {
-  scm_misc_error (funcname, "the field is invalid", SCM_BOOL_F);
+  scm_misc_error (funcname, "the field is invalid", SCM_EOL);
 }
 
 void
 form_no_room_error (const char *funcname)
 {
   scm_misc_error (funcname, "the form or field can not fit on this screen",
-		  SCM_BOOL_F);
+                  SCM_BOOL_F);
 }
 
 void
 form_not_connected_error (const char *funcname)
 {
-  scm_misc_error (funcname, "the field is not connected to a form", SCM_BOOL_F);
+  scm_misc_error (funcname, "the field is not connected to a form", SCM_EOL);
 }
 
 void
 form_not_posted_error (const char *funcname)
 {
-  scm_misc_error (funcname, "the form is not posted", SCM_BOOL_F);
+  scm_misc_error (funcname, "the form is not posted", SCM_EOL);
 }
 
 void
 form_posted_error (const char *funcname)
 {
-  scm_misc_error (funcname, "the form is posted", SCM_BOOL_F);
+  scm_misc_error (funcname, "the form is posted", SCM_EOL);
 }
 
 /* Given a form, return the current field. */
@@ -105,7 +104,8 @@ gucu_current_field (SCM form)
   FIELD *c_field = current_field (c_form);
   if (c_field != (FIELD *) 0)
     if (!field_increase_refcount (c_field))
-      scm_misc_error ("current-field", "too many references to field", NULL);
+      scm_misc_error ("current-field", "too many references to field",
+                      SCM_EOL);
   SCM s_field = _scm_from_field (c_field);
 
   return s_field;
@@ -292,7 +292,7 @@ gucu_form_driver (SCM form, SCM c)
 {
   SCM_ASSERT (_scm_is_form (form), form, SCM_ARG1, "form-driver");
   SCM_ASSERT (SCM_CHARP (c) || scm_is_integer (c), c, SCM_ARG2,
-	      "form-driver");
+              "form-driver");
 
   FORM *c_form = _scm_to_form (form);
   int ret;
@@ -444,36 +444,21 @@ gucu_form_request_name (SCM req)
 SCM
 gucu_form_sub (SCM form)
 {
-  struct gucu_form *gf;
-
-  scm_assert_smob_type (form_tag, form);
-
-  gf = (struct gucu_form *) SCM_SMOB_DATA (form);
-
+  scm_assert_foreign_object_type (form_fo_type, form);
   /* Return the subwindow, if one has already been assigned */
-  if (scm_is_true (gf->sub_guard))
-    return gf->sub_guard;
-
-  /* Otherwise, the subwindow is stdscr */
-  return SCM_BOOL_F;
+  /* Otherwise, the subwindow is stdscr. */
+  return SCM_PACK_POINTER (scm_foreign_object_ref (form, 2));
 }
 
 /* Return the main window of this form */
 SCM
 gucu_form_win (SCM form)
 {
-  struct gucu_form *gf;
-
-  scm_assert_smob_type (form_tag, form);
-
-  gf = (struct gucu_form *) SCM_SMOB_DATA (form);
+  scm_assert_foreign_object_type (form_fo_type, form);
 
   /* Return the window, if one has already been assigned */
-  if (scm_is_true (gf->win_guard))
-    return gf->win_guard;
-
-  /* Otherwise, the subwindow is stdscr */
-  return SCM_BOOL_F;
+  /* Otherwise, the window is stdscr. */
+  return SCM_PACK_POINTER (scm_foreign_object_ref (form, 1));
 }
 
 /* Manually free the storage associated with the field */
@@ -794,17 +779,13 @@ gucu_set_form_page_x (SCM form, SCM n)
 SCM
 gucu_set_form_sub_x (SCM form, SCM win)
 {
-  struct gucu_form *gf;
-
-  scm_assert_smob_type (form_tag, form);
-
-  gf = (struct gucu_form *) SCM_SMOB_DATA (form);
+  scm_assert_foreign_object_type (form_fo_type, form);
 
   SCM_ASSERT (_scm_is_window (win), win, SCM_ARG2, "set-form-sub!");
 
   WINDOW *c_win = _scm_to_window (win);
 
-  int ret = set_form_sub (gf->form, c_win);
+  int ret = set_form_sub (scm_foreign_object_ref (form, 0), c_win);
 
   if (ret == E_BAD_ARGUMENT)
     scm_out_of_range ("set-form-sub!", win);
@@ -818,9 +799,7 @@ gucu_set_form_sub_x (SCM form, SCM win)
   /* If this is stdscr, we shouldn't store it as SUB, because it could
      be returned by form_sub and then possibly freed */
   if (c_win != stdscr)
-    {
-      gf->sub_guard = win;
-    }
+    scm_foreign_object_set_x (form, 2, SCM_UNPACK_POINTER (win));
 
   return SCM_UNSPECIFIED;
 }
@@ -829,17 +808,13 @@ gucu_set_form_sub_x (SCM form, SCM win)
 SCM
 gucu_set_form_win_x (SCM form, SCM win)
 {
-  struct gucu_form *gf;
-
-  scm_assert_smob_type (form_tag, form);
-
-  gf = (struct gucu_form *) SCM_SMOB_DATA (form);
+  scm_assert_foreign_object_type (form_fo_type, form);
 
   SCM_ASSERT (_scm_is_window (win), win, SCM_ARG2, "set-form-win!");
 
   WINDOW *c_win = _scm_to_window (win);
 
-  int ret = set_form_win (gf->form, c_win);
+  int ret = set_form_win (scm_foreign_object_ref (form, 0), c_win);
 
   if (ret == E_BAD_ARGUMENT)
     scm_out_of_range ("set-form-win!", win);
@@ -853,9 +828,7 @@ gucu_set_form_win_x (SCM form, SCM win)
   /* If this is stdscr, we shouldn't store it as WIN, because it could
      be returned by form_win and then possibly freed */
   if (c_win != stdscr)
-    {
-      gf->win_guard = win;
-    }
+    scm_foreign_object_set_x (form, 1, SCM_UNPACK_POINTER (win));
 
   return SCM_UNSPECIFIED;
 }
@@ -951,7 +924,7 @@ gucu_form_init_function ()
   scm_c_define_gsubr ("form-opts-off!", 2, 0, 0, gucu_form_opts_off_x);
   scm_c_define_gsubr ("form-page", 1, 0, 0, gucu_form_page);
   scm_c_define_gsubr ("form-request-by-name", 1, 0, 0,
-		      gucu_form_request_by_name);
+                      gucu_form_request_by_name);
   scm_c_define_gsubr ("form-request-name", 1, 0, 0, gucu_form_request_name);
   scm_c_define_gsubr ("form-sub", 1, 0, 0, gucu_form_sub);
   scm_c_define_gsubr ("form-win", 1, 0, 0, gucu_form_win);
@@ -962,7 +935,7 @@ gucu_form_init_function ()
   scm_c_define_gsubr ("pos-form-cursor", 1, 0, 0, gucu_pos_form_cursor);
   scm_c_define_gsubr ("post-form", 1, 0, 0, gucu_post_form);
   scm_c_define_gsubr ("set-current-field!", 2, 0, 0,
-		      gucu_set_current_field_x);
+                      gucu_set_current_field_x);
   scm_c_define_gsubr ("set-field-back!", 2, 0, 0, gucu_set_field_back_x);
   scm_c_define_gsubr ("set-field-buffer!", 3, 0, 0, gucu_set_field_buffer_x);
   scm_c_define_gsubr ("set-field-fore!", 2, 0, 0, gucu_set_field_fore_x);
