@@ -1,7 +1,7 @@
 /*
   form_type.c
 
-  Copyright 2009, 2010, 2011, 2014, 2016 Free Software Foundation, Inc.
+  Copyright 2009, 2010, 2011, 2014, 2016, 2019 Free Software Foundation, Inc.
 
   This file is part of GNU Guile-Ncurses.
 
@@ -40,28 +40,19 @@
 #error "No curses.h file included"
 #endif
 
-#include "compat.h"
 #include "form_func.h"
 #include "form_type.h"
 #include "gucuconfig.h"
 #include "type.h"
 
-scm_t_bits form_tag;
-scm_t_bits field_tag;
+SCM form_fo_type;
+SCM field_fo_type;
 
-SCM equalp_field (SCM x1, SCM x2);
-size_t gc_free_field (SCM x);
-SCM mark_field (SCM x);
-int print_field (SCM x, SCM port, scm_print_state * pstate);
+void gc_free_field (SCM x);
+void gc_free_form (SCM x);
 
-
-SCM equalp_form (SCM x1, SCM x2);
-size_t gc_free_form (SCM x);
-SCM mark_form (SCM x);
-int print_form (SCM x, SCM port, scm_print_state * pstate);
-
-
-// field -- in C, a FIELD.  In Scheme, a smob that contains the pointer
+// field -- in C, a FIELD.  In Scheme, a foreign object that contains
+// the pointer
 
 void
 field_init_refcount (FIELD *field)
@@ -100,9 +91,9 @@ field_get_refcount (FIELD *field)
 int
 _scm_is_field (SCM x)
 {
-  if (SCM_SMOB_PREDICATE (field_tag, x))
+  if (SCM_IS_A_P (x, field_fo_type))
     {
-      if (SCM_SMOB_DATA (x) != 0)
+      if (scm_foreign_object_ref (x, 0) != NULL)
         return 1;
       else
         return 0;
@@ -114,43 +105,29 @@ _scm_is_field (SCM x)
 FIELD *
 _scm_to_field (SCM x)
 {
-  return (FIELD *) SCM_SMOB_DATA (x);
+  return (FIELD *) scm_foreign_object_ref (x, 0);
 }
 
 SCM
-_scm_from_field (FIELD * x)
+_scm_from_field (FIELD *x)
 {
   SCM s_field;
 
   assert (x != NULL);
 
-  SCM_NEWSMOB (s_field, field_tag, x);
+  s_field = scm_make_foreign_object_1 (field_fo_type, x);
 
-  assert (x == (FIELD *) SCM_SMOB_DATA (s_field));
+  assert (x == (FIELD *) scm_foreign_object_ref (s_field, 0));
 
   if (0)
     {
-      fprintf (stderr, "Making <#field> smob from FIELD * %p\n", (void *) x);
+      fprintf (stderr, "Making <#field> foreign object from FIELD * %p\n",
+               (void *) x);
     }
 
   return (s_field);
 }
 
-// Fields are equal if they point to the same C structure
-SCM
-equalp_field (SCM x1, SCM x2)
-{
-  FIELD *field1 = (FIELD *) SCM_SMOB_DATA (x1);
-  FIELD *field2 = (FIELD *) SCM_SMOB_DATA (x2);
-
-  if ((field1 == NULL) || (field2 == NULL))
-    return SCM_BOOL_F;
-  else if ((field1 != field2))
-    return SCM_BOOL_F;
-  else
-    return SCM_BOOL_T;
-}
-
 SCM
 gucu_new_field (SCM height, SCM width, SCM top, SCM left, SCM offscreen,
                 SCM nbuffers)
@@ -194,17 +171,10 @@ gucu_new_field (SCM height, SCM width, SCM top, SCM left, SCM offscreen,
 }
 
 
-SCM
-mark_field (SCM x UNUSED)
-{
-  // No SCMs in the field type: nothing to do here.
-  return (SCM_BOOL_F);
-}
-
 /* The name is free_field.  The curses primitive that frees memory is
    called del_field. Note that del_field doesn't free the underlying
    window. */
-size_t
+void
 gc_free_field (SCM field)
 {
   FIELD *f = _scm_to_field (field);
@@ -213,44 +183,18 @@ gc_free_field (SCM field)
     {
       field_decrease_refcount (f);
       if (field_get_refcount (f) == 0)
-	{
-	  /* No other #<field> or #<form> is using this field, so we
-	     can free its data.  */
-	  int ret = free_field (f);
-	  if (ret != E_OK)
-	    {
-	      /* ??? --- what should happen when free fails? */
-	    }
-	}
+        {
+          /* No other #<field> or #<form> is using this field, so we
+             can free its data.  */
+          int ret = free_field (f);
+          if (ret != E_OK)
+            {
+              /* ??? --- what should happen when free fails? */
+            }
+        }
       /* Detach the field's data from the field.  */
-      SCM_SET_SMOB_DATA (field, NULL);
+      scm_foreign_object_set_x (field, 0, NULL);
     }
-
-  return 0;
-}
-
-int
-print_field (SCM x, SCM port, scm_print_state * pstate UNUSED)
-{
-  FIELD *fld = (FIELD *) SCM_SMOB_DATA (x);
-  char str[SIZEOF_VOID_P*2+3];
-
-  scm_puts ("#<field ", port);
-
-  if (fld == 0)
-    scm_puts ("(freed)", port);
-  else
-    {
-      if (snprintf (str, sizeof(str), "%p", (void *) fld) < 0)
-        scm_puts ("???", port);
-      else
-        scm_puts (str, port);
-    }
-
-  scm_puts (">", port);
-
-  // non-zero means success
-  return 1;
 }
 
 SCM
@@ -278,12 +222,12 @@ _scm_is_list_of_fields (SCM fields)
   len = scm_to_size_t (scm_length (fields));
   if (len == 0)
     return 0;
-  for (i = 0; i < len; i ++)
+  for (i = 0; i < len; i++)
     {
       SCM entry;
       entry = scm_list_ref (fields, scm_from_int (i));
       if (!_scm_is_field (entry))
-	return 0;
+        return 0;
     }
   return 1;
 }
@@ -298,20 +242,21 @@ _scm_is_list_of_unattached_fields (SCM fields)
   len = scm_to_size_t (scm_length (fields));
   if (len == 0)
     return 0;
-  for (i = 0; i < len; i ++)
+  for (i = 0; i < len; i++)
     {
       SCM entry;
       entry = scm_list_ref (fields, scm_from_int (i));
       if (!_scm_is_field (entry))
-	return 0;
+        return 0;
       if (field_index (_scm_to_field (entry)) != ERR)
-	return 0;
+        return 0;
     }
   return 1;
 }
 
-// form -- in C, a FORM *.  In Scheme, a smob that contains the pointer
-// to a form along with a list that contains the SCM of the fields
+// form -- in C, a FORM *.  In Scheme, a foreign object that contains
+// the pointer to a form along with a list that contains the SCM of
+// the fields
 
 // N.B.: form->field must point to a C array containing the FIELD *
 // contained in the SCM fields.
@@ -319,120 +264,60 @@ _scm_is_list_of_unattached_fields (SCM fields)
 int
 _scm_is_form (SCM x)
 {
-  return SCM_SMOB_PREDICATE (form_tag, x);
+  return SCM_IS_A_P (x, form_fo_type);
 }
 
 FORM *
 _scm_to_form (SCM x)
 {
-  struct gucu_form *gf;
-
-  scm_assert_smob_type (form_tag, x);
-
-  gf = (struct gucu_form *) SCM_SMOB_DATA (x);
-
-  return (FORM *) gf->form;
+  scm_assert_foreign_object_type (form_fo_type, x);
+  return (FORM *) scm_foreign_object_ref (x, 0);
 }
 
-
-// Forms are equal if they point to the same C structure
-SCM
-equalp_form (SCM x1, SCM x2)
-{
-  FORM *form1 = (FORM *) SCM_SMOB_DATA (x1);
-  FORM *form2 = (FORM *) SCM_SMOB_DATA (x2);
-
-  if ((form1 == NULL) || (form2 == NULL))
-    return SCM_BOOL_F;
-  else if ((form1 != form2))
-    return SCM_BOOL_F;
-  else
-    return SCM_BOOL_T;
-}
-
-SCM
-mark_form (SCM x)
-{
-  struct gucu_form *gf;
-
-  gf = (struct gucu_form *) SCM_SMOB_DATA (x);
-  if (gf != NULL)
-    {
-      scm_gc_mark (gf->win_guard);
-      scm_gc_mark (gf->sub_guard);
-    }
-
-  return SCM_BOOL_F;
-}
-
-size_t
+void
 gc_free_form (SCM x)
 {
-  struct gucu_form *form;
   int retval;
 
-  form = (struct gucu_form *) SCM_SMOB_DATA (x);
+  FORM *form = (FORM *) scm_foreign_object_ref (x, 0);
 
-  if (form != NULL && form->form != NULL)
+  if (form != NULL)
     {
-      FIELD **pfields = form_fields (form->form);
+      int len = field_count (form);
+      FIELD **pfields = form_fields (form);
 
-      retval = free_form (form->form);
-      form->form = (FORM *) NULL;
+      retval = free_form (form);
+      scm_foreign_object_set_x (x, 0, NULL);
       if (retval == E_BAD_ARGUMENT)
         {
           scm_error_scm (scm_from_locale_symbol ("ncurses"),
-                         scm_from_locale_string ("garbage collection of form"),
-                         scm_from_locale_string ("bad argument"),
-                         SCM_BOOL_F, SCM_BOOL_F);
+                         scm_from_locale_string
+                         ("garbage collection of form"),
+                         scm_from_locale_string ("bad argument"), SCM_BOOL_F,
+                         SCM_BOOL_F);
         }
       else if (retval == E_POSTED)
         {
           scm_error_scm (scm_from_locale_symbol ("ncurses"),
-                         scm_from_locale_string ("garbage collection of form"),
-                         scm_from_locale_string ("posted"),
-                         SCM_BOOL_F, SCM_BOOL_F);
+                         scm_from_locale_string
+                         ("garbage collection of form"),
+                         scm_from_locale_string ("posted"), SCM_BOOL_F,
+                         SCM_BOOL_F);
         }
 
       /* Decrease the refcount and maybe free the fields.  */
-      while (*pfields != NULL)
-	{
-	  field_decrease_refcount (*pfields);
-	  if (field_get_refcount (*pfields) == 0)
-	    free_field (*pfields);
-	  pfields ++;
-	}
+      for (int i = 0; i < len; i++)
+        {
+          field_decrease_refcount (pfields[i]);
+          if (field_get_refcount (pfields[i]) == 0)
+            free_field (pfields[i]);
+        }
+      free (pfields);
 
       /* Release the hold on any windows.  */
-      form->win_guard = SCM_BOOL_F;
-      form->sub_guard = SCM_BOOL_F;
+      scm_foreign_object_set_x (x, 1, SCM_UNPACK_POINTER (SCM_BOOL_F));
+      scm_foreign_object_set_x (x, 2, SCM_UNPACK_POINTER (SCM_BOOL_F));
     }
-  SCM_SET_SMOB_DATA (x, NULL);
-
-  return 0;
-}
-
-int
-print_form (SCM x, SCM port, scm_print_state * pstate UNUSED)
-{
-  struct gucu_form *frm = (struct gucu_form *) SCM_SMOB_DATA (x);
-  char str[SIZEOF_VOID_P*2+3];
-
-  assert (frm != NULL);
-
-  scm_puts ("#<form ", port);
-
-  if (frm == NULL)
-    scm_puts ("(freed)", port);
-  else if (snprintf (str, sizeof(str), "%p", (void *) frm->form) < 0)
-    scm_puts ("???", port);
-  else
-    scm_puts (str, port);
-
-  scm_puts (">", port);
-
-  // non-zero means success
-  return 1;
 }
 
 SCM
@@ -441,14 +326,12 @@ gucu_is_form_p (SCM x)
   return scm_from_bool (_scm_is_form (x));
 }
 
-
 SCM
 gucu_new_form (SCM fields)
 {
-  struct gucu_form *gf;
   size_t len;
   FIELD **c_fields;
-  SCM smob;
+  SCM fobj;
   SCM entry;
   size_t i;
 
@@ -457,21 +340,19 @@ gucu_new_form (SCM fields)
     scm_wrong_type_arg_msg ("new-form", SCM_ARG1, fields, "list of #<field>");
   if (!_scm_is_list_of_unattached_fields (fields))
     scm_misc_error ("new-form",
-		    "fields may not be attached to more than one form: ~s",
-		    scm_list_1 (fields));
+                    "fields may not be attached to more than one form: ~s",
+                    scm_list_1 (fields));
 
   // Step 1: allocate memory
   len = scm_to_size_t (scm_length (fields));
-  gf = scm_gc_malloc (sizeof (struct gucu_form), "gucu_form");
-  c_fields = scm_gc_malloc (sizeof (FIELD *) * (len + 1), "gucu_form");
+  c_fields = malloc (sizeof (FIELD *) * (len + 1));
 
   // Step 2: initialize it with C code
-  gf->form = 0;
-  gf->win_guard = SCM_BOOL_F;
-  gf->sub_guard = SCM_BOOL_F;
-
-  // Step 3: Create the smob
-  SCM_NEWSMOB (smob, form_tag, gf);
+  // Step 3: Create the foreign object
+  fobj =
+    scm_make_foreign_object_3 (form_fo_type, NULL,
+                               SCM_UNPACK_POINTER (SCM_BOOL_F),
+                               SCM_UNPACK_POINTER (SCM_BOOL_F));
 
   // Step 4: Finish the initialization
   for (i = 0; i < len; i++)
@@ -481,9 +362,9 @@ gucu_new_form (SCM fields)
     }
   c_fields[len] = (FIELD *) NULL;
 
-  gf->form = new_form (c_fields);
+  FORM *form = new_form (c_fields);
 
-  if (gf->form == NULL)
+  if (form == NULL)
     {
       free (c_fields);
       if (errno == E_BAD_ARGUMENT)
@@ -516,46 +397,46 @@ gucu_new_form (SCM fields)
       entry = scm_list_ref (fields, scm_from_int (i));
       field_increase_refcount (c_fields[i]);
     }
+  scm_foreign_object_set_x (fobj, 0, form);
 
-  return smob;
+  return fobj;
 }
 
 // Return the fields on which the form depends
 SCM
 gucu_form_fields (SCM form)
 {
-  struct gucu_form *gf;
   int i;
 
-  scm_assert_smob_type (form_tag, form);
+  scm_assert_foreign_object_type (form_fo_type, form);
 
-  gf = (struct gucu_form *) SCM_SMOB_DATA (form);
-  if (gf == NULL || gf->form == NULL)
+  FORM *c_form = (FORM *) scm_foreign_object_ref (form, 0);
+  if (c_form == NULL)
     return SCM_EOL;
   else
     {
-      int len = field_count (gf->form);
+      int len = field_count (c_form);
       if (len == ERR || len == 0)
-	return SCM_EOL;
+        return SCM_EOL;
       else
-	{
-	  FIELD **pfields;
-	  pfields = form_fields (gf->form);
-	  SCM list = SCM_EOL;
-	  if (pfields == NULL)
-	    return SCM_EOL;
-	  else
-	    {
-	      for (i = 0; i < len; i ++)
-		{
-		  SCM entry;
-		  field_increase_refcount (pfields[i]);
-		  entry = _scm_from_field (pfields[i]);
-		  list = scm_append (scm_list_2 (list, scm_list_1 (entry)));
-		}
-	      return list;
-	    }
-	}
+        {
+          FIELD **pfields;
+          pfields = form_fields (c_form);
+          SCM list = SCM_EOL;
+          if (pfields == NULL)
+            return SCM_EOL;
+          else
+            {
+              for (i = 0; i < len; i++)
+                {
+                  SCM entry;
+                  field_increase_refcount (pfields[i]);
+                  entry = _scm_from_field (pfields[i]);
+                  list = scm_append (scm_list_2 (list, scm_list_1 (entry)));
+                }
+              return list;
+            }
+        }
     }
   return SCM_EOL;
 }
@@ -567,87 +448,85 @@ gucu_set_form_fields_x (SCM form, SCM fields)
 {
   SCM_ASSERT (_scm_is_form (form), form, SCM_ARG1, "set-form-fields!");
 
-  struct gucu_form *gf;
   FIELD **c_fields;
   SCM entry;
   size_t i;
   int ret;
 
   if (!_scm_is_list_of_fields (fields))
-    scm_wrong_type_arg_msg ("set-form-field!", SCM_ARG2, fields, "list of #<field>");
+    scm_wrong_type_arg_msg ("set-form-field!", SCM_ARG2, fields,
+                            "list of #<field>");
 
   /* FIXME: Check that all of the fields are either unattached or are attached to this
      form.  Is there a way to do that check?  */
 
-  gf = (struct gucu_form *) SCM_SMOB_DATA (form);
-  if (gf != NULL && gf->form != NULL)
+  FORM *c_form = (FORM *) scm_foreign_object_ref (form, 0);
+  if (c_form != NULL)
     {
       FIELD **pfield_prev = NULL;
       int len_prev;
       int len_cur;
 
       /* Hold on to the old fields list.  */
-      len_prev = field_count (gf->form);
-      pfield_prev = form_fields (gf->form);
+      len_prev = field_count (c_form);
+      pfield_prev = form_fields (c_form);
 
       /* Make a new fields list.  */
       len_cur = scm_to_int (scm_length (fields));
-      c_fields = scm_gc_malloc (sizeof (FIELD *) * (len_cur + 1), "set-form-fields!");
+      c_fields = malloc (sizeof (FIELD *) * (len_cur + 1));
       for (i = 0; i < len_cur; i++)
-	{
-	  entry = scm_list_ref (fields, scm_from_int (i));
-	  c_fields[i] = _scm_to_field (entry);
-	}
+        {
+          entry = scm_list_ref (fields, scm_from_int (i));
+          c_fields[i] = _scm_to_field (entry);
+        }
       c_fields[len_cur] = (FIELD *) NULL;
 
       /* Attach the new fields to the form.  */
-      ret = set_form_fields (gf->form, c_fields);
+      ret = set_form_fields (c_form, c_fields);
       if (ret == E_BAD_ARGUMENT)
-	scm_out_of_range ("set-form-fields!", fields);
+        scm_out_of_range ("set-form-fields!", fields);
       else if (ret == E_CONNECTED)
-	form_connected_error ("set-form-fields!");
+        form_connected_error ("set-form-fields!");
       else if (ret == E_POSTED)
-	form_posted_error ("set-form-fields!");
+        form_posted_error ("set-form-fields!");
       else if (ret == E_SYSTEM_ERROR)
-	scm_syserror ("set-form-fields!");
+        scm_syserror ("set-form-fields!");
 
       /* Increase the refcount on the new fields.  */
       for (i = 0; i < len_cur; i++)
-	field_increase_refcount (c_fields[i]);
+        field_increase_refcount (c_fields[i]);
 
       /* Decrease the refcount on the old fields, and maybe free. */
       if (len_prev > 0 && pfield_prev != NULL)
-	{
-	  for (i = 0; i < len_prev; i ++)
-	    {
-	      field_decrease_refcount (pfield_prev[i]);
-	      if (field_get_refcount (pfield_prev[i]) == 0)
-		free_field (pfield_prev[i]);
-	    }
-	}
+        {
+          for (i = 0; i < len_prev; i++)
+            {
+              field_decrease_refcount (pfield_prev[i]);
+              if (field_get_refcount (pfield_prev[i]) == 0)
+                free_field (pfield_prev[i]);
+            }
+          free (pfield_prev);
+        }
     }
 
   return SCM_UNSPECIFIED;
 }
 
 
+#define makeFO(a,b,c) scm_make_foreign_object_type((a),(b),(c))
+#define u8sym(x) scm_from_utf8_symbol(x)
 void
 gucu_form_init_type ()
 {
-  field_tag = scm_make_smob_type ("field", sizeof (FIELD *));
-  scm_set_smob_mark (field_tag, mark_field);
-  scm_set_smob_free (field_tag, gc_free_field);
-  scm_set_smob_print (field_tag, print_field);
-  scm_set_smob_equalp (field_tag, equalp_field);
+  field_fo_type = makeFO (u8sym ("field"),
+                          scm_list_1 (u8sym ("field")), gc_free_field);
   scm_c_define_gsubr ("field?", 1, 0, 0, gucu_is_field_p);
   scm_c_define_gsubr ("%field-refcount", 1, 0, 0, gucu_field_refcount);
 
-
-  form_tag = scm_make_smob_type ("form", sizeof (FORM *));
-  scm_set_smob_mark (form_tag, mark_form);
-  scm_set_smob_free (form_tag, gc_free_form);
-  scm_set_smob_print (form_tag, print_form);
-  scm_set_smob_equalp (form_tag, equalp_form);
+  form_fo_type = makeFO (u8sym ("form"),
+                         scm_list_3 (u8sym ("form"),
+                                     u8sym ("window"),
+                                     u8sym ("subwindow")), gc_free_form);
   scm_c_define_gsubr ("form?", 1, 0, 0, gucu_is_form_p);
 
   scm_c_define_gsubr ("new-field", 6, 0, 0, gucu_new_field);
