@@ -1,7 +1,7 @@
 /*
   type.c
 
-  Copyright 2009, 2010, 2011, 2014, 2016 Free Software Foundation, Inc.
+  Copyright 2009, 2010, 2011, 2014, 2016, 2019 Free Software Foundation, Inc.
 
   This file is part of GNU Guile-Ncurses.
 
@@ -40,7 +40,6 @@
 #error "No curses.h file included"
 #endif
 
-#include "compat.h"
 #include "gucuconfig.h"
 #include "type.h"
 #include "unicode.h"
@@ -52,17 +51,13 @@
 #define GUCU_CCHARW_MAX (5)
 #endif
 
-static scm_t_bits screen_tag;
-scm_t_bits window_tag;
+static SCM screen_fo_type;
+SCM window_fo_type;
 
-static size_t gc_free_screen (SCM x);
-static int print_screen (SCM x, SCM port, scm_print_state * pstate);
-
-static SCM equalp_window (SCM x1, SCM x2);
-static size_t gc_free_window (SCM x);
-static SCM mark_window (SCM x);
-static int print_window (SCM x, SCM port, scm_print_state * pstate);
+static void gc_free_screen (SCM x);
 
+static SCM gucu_window_equalp (SCM x1, SCM x2);
+static void gc_free_window (SCM x);
 
 /* attr -- character attributes, bit flags packed into an unsigned:
    probably uint32 */
@@ -528,7 +523,7 @@ _scm_sstring_from_wint_string (const wint_t * x)
 
 #ifdef HAVE_NCURSESW
 SCM
-_scm_sstring_from_wstring (const wchar_t * x)
+_scm_sstring_from_wstring (const wchar_t *x)
 {
   size_t i;
   SCM member, xstring;
@@ -579,7 +574,7 @@ _scm_sstring_to_wstring (SCM x)
 }
 
 SCM
-_scm_xstring_from_chstring (const chtype * x)
+_scm_xstring_from_chstring (const chtype *x)
 {
   size_t i;
   SCM member, xstring;
@@ -848,19 +843,12 @@ gucu_is_mevent_p (SCM x)
 // garbage collected prematurely, we carry references to them along
 // with the SCREEN structure.  They must all exist as a set.
 
-struct screen_and_ports
-{
-  SCREEN *screen;               /* the SCREEN* created by ncurses newterm */
-  FILE *ifp;                    /* The associated input file stream */
-  FILE *ofp;                    /* The associated output file stream */
-};
-
 int
 _scm_is_screen (SCM x)
 {
-  if (SCM_SMOB_PREDICATE (screen_tag, x))
+  if (SCM_IS_A_P (x, screen_fo_type))
     {
-      if (SCM_SMOB_DATA (x) != 0)
+      if (scm_foreign_object_ref (x, 0) != NULL)
         return 1;
       else
         return 0;
@@ -873,127 +861,67 @@ SCREEN *
 _scm_to_screen (SCM x)
 {
   assert (_scm_is_screen (x));
-  struct screen_and_ports *sp = (struct screen_and_ports *) SCM_SMOB_DATA (x);
-  return sp->screen;
+  return scm_foreign_object_ref (x, 0);
 }
 
 void
-_scm_to_screen_and_ports (SCM x, SCREEN ** screen, FILE ** ofp, FILE ** ifp)
+_scm_to_screen_and_ports (SCM x, SCREEN **screen, FILE **ifp, FILE **ofp)
 {
   assert (_scm_is_screen (x));
   assert (screen != NULL);
-  assert (ofp != NULL);
   assert (ifp != NULL);
-  struct screen_and_ports *sp = (struct screen_and_ports *) SCM_SMOB_DATA (x);
-  *screen = sp->screen;
-  *ofp = sp->ofp;
-  *ifp = sp->ifp;
+  assert (ofp != NULL);
+  *screen = scm_foreign_object_ref (x, 0);
+  *ifp = scm_foreign_object_ref (x, 1);
+  *ofp = scm_foreign_object_ref (x, 2);
 }
 
 SCM
-_scm_from_screen_and_ports (SCREEN * x, FILE * ofp, FILE * ifp)
+_scm_from_screen_and_ports (SCREEN *x, FILE *ifp, FILE *ofp)
 {
   SCM s_screen;
 
   assert (x != NULL);
-  assert (ofp != NULL);
   assert (ifp != NULL);
+  assert (ofp != NULL);
 
-  struct screen_and_ports *sp = scm_malloc (sizeof (struct screen_and_ports));
-  sp->screen = x;
-  sp->ofp = ofp;
-  sp->ifp = ifp;
-  SCM_NEWSMOB (s_screen, screen_tag, sp);
+  s_screen = scm_make_foreign_object_3 (screen_fo_type, x, ifp, ofp);
   return s_screen;
 }
 
+/*  This procedure frees a screen without checking its type. In GC
+ *  free procedures, the foreign object may have already forgotten its
+ *  type. */
+static void
+gc_free_screen (SCM x)
+{
+  SCREEN *screen = scm_foreign_object_ref (x, 0);
+  FILE *ifp = scm_foreign_object_ref (x, 1);
+  FILE *ofp = scm_foreign_object_ref (x, 2);
+
+  if (screen != NULL)
+    {
+      delscreen (screen);
+      /* delscreen returns void */
+      scm_foreign_object_set_x (x, 0, NULL);
+    }
+  if (ifp)
+    {
+      fclose (ifp);
+      scm_foreign_object_set_x (x, 1, NULL);
+    }
+  if (ofp)
+    {
+      fclose (ofp);
+      scm_foreign_object_set_x (x, 2, NULL);
+    }
+}
+
 void
 _scm_free_screen (SCM x)
 {
   assert (_scm_is_screen (x));
-  struct screen_and_ports *sp = (struct screen_and_ports *) SCM_SMOB_DATA (x);
-
-  if (sp->screen)
-    {
-      delscreen (sp->screen);
-      /* delscreen returns void */
-      sp->screen = NULL;
-    }
-  if (sp->ifp)
-    {
-      fclose (sp->ifp);
-      sp->ifp = NULL;
-    }
-  if (sp->ofp)
-    {
-      fclose (sp->ofp);
-      sp->ofp = NULL;
-    }
-}
-
-/*  This procedure frees a screen without checking its type. In GC
- *  free procedures, the smob may have already forgotten its type. */
-static size_t
-gc_free_screen (SCM x)
-{
-  struct screen_and_ports *sp = (struct screen_and_ports *) SCM_SMOB_DATA (x);
-
-  /* Screens should already be null if delwin has been called on them */
-  if (sp != NULL)
-    {
-      if (sp->screen)
-        {
-          delscreen (sp->screen);
-          /* delscreen returns void */
-          sp->screen = NULL;
-        }
-      if (sp->ifp)
-        {
-          fclose (sp->ifp);
-          sp->ifp = NULL;
-        }
-      if (sp->ofp)
-        {
-          fclose (sp->ofp);
-          sp->ofp = NULL;
-        }
-      free (sp);
-      SCM_SET_SMOB_DATA (x, 0);
-    }
-
-  return 0;
-}
-
-static int
-print_screen (SCM x, SCM port, scm_print_state * pstate UNUSED)
-{
-  struct screen_and_ports *sp;
-  SCREEN *screen;
-  char str[SIZEOF_VOID_P * 2 + 3];
-
-  /* A function that may operated on GC'd #<screens>
-   * shouldn't typecheck them, so no typecheck assert here. */
-
-  sp = (struct screen_and_ports *) SCM_SMOB_DATA (x);
-  screen = sp->screen;
-  scm_puts ("#<screen ", port);
-
-  if (screen == NULL)
-    {
-      scm_puts ("(freed)", port);
-    }
-  else
-    {
-      if (snprintf (str, sizeof (str), "%p", (void *) screen) < 0)
-        scm_puts ("???", port);
-      else
-        scm_puts (str, port);
-    }
-
-  scm_puts (">", port);
-
-  // non-zero means success
-  return 1;
+  gc_free_screen (x);
 }
 
 SCM
@@ -1002,14 +930,21 @@ gucu_is_screen_p (SCM x)
   return scm_from_bool (_scm_is_screen (x));
 }
 
-// window -- in C, a struct gucu_window *.  In Scheme, a smob that contains the pointer
+SCM
+gucu_screen_equalp (SCM x, SCM y)
+{
+  return scm_from_bool (_scm_to_screen (x) == _scm_to_screen (y));
+}
+
+// window -- in C, a WINDOW *.  In Scheme, a foreign
+// object that contains the pointer
 
 int
 _scm_is_window (SCM x)
 {
-  if (SCM_SMOB_PREDICATE (window_tag, x))
+  if (SCM_IS_A_P (x, window_fo_type))
     {
-      if (SCM_SMOB_DATA (x) != 0)
+      if (scm_foreign_object_ref (x, 0) != NULL)
         return 1;
       else
         return 0;
@@ -1022,42 +957,32 @@ _scm_is_window (SCM x)
 WINDOW *
 _scm_to_window (SCM x)
 {
-  struct gucu_window *wp = NULL;
   assert (_scm_is_window (x));
 
-  wp = (struct gucu_window *) SCM_SMOB_DATA (x);
-  if (wp != (struct gucu_window *) NULL)
-    return wp->window;
-
-  return (WINDOW *) NULL;
+  return scm_foreign_object_ref (x, 0);
 }
 
 SCM
-_scm_from_window_full (SCM parent, SCM name, WINDOW * win)
+_scm_from_window_full (SCM parent, SCM name, WINDOW *win)
 {
   SCM s_win;
-  struct gucu_window *wp;
 
   assert (win != NULL);
-  assert (_scm_is_window (parent) || parent == SCM_BOOL_F);
-  assert (scm_is_string (name) || name == SCM_BOOL_F);
+  assert (_scm_is_window (parent) || scm_is_false (parent));
+  assert (scm_is_string (name) || scm_is_false (name));
+  void *vals[4] = { win, NULL, SCM_UNPACK_POINTER (parent),
+    SCM_UNPACK_POINTER (name)
+  };
 
-  wp = scm_gc_malloc (sizeof (struct gucu_window), "_scm_from_window_full");
-
-  SCM_NEWSMOB (s_win, window_tag, wp);
-
-  wp->parent = parent;
-  wp->name = name;
-  wp->window = win;
-  wp->panel = (PANEL *) NULL;
+  s_win = scm_make_foreign_object_n (window_fo_type, 4, vals);
 
   if (0)
     {
 #ifdef NCURSES_OPAQUE
-      fprintf (stderr, "Making smob from window\n");
+      fprintf (stderr, "Making foreign object from window\n");
 #else
-      fprintf (stderr, "Making smob from window at %d, %d\n", x->_begx,
-               x->_begy);
+      fprintf (stderr, "Making foreign object from window at %d, %d\n",
+               x->_begx, x->_begy);
 #endif
     }
 
@@ -1065,22 +990,19 @@ _scm_from_window_full (SCM parent, SCM name, WINDOW * win)
 }
 
 SCM
-_scm_from_window (WINDOW * x)
+_scm_from_window (WINDOW *x)
 {
   return _scm_from_window_full (SCM_BOOL_F, SCM_BOOL_F, x);
 }
 
 // Windows are equal if they point to the same C structures
 static SCM
-equalp_window (SCM x1, SCM x2)
+gucu_window_equalp (SCM x1, SCM x2)
 {
   WINDOW *win1, *win2;
 
-  /* This assert is thrown if x1 or x2 are already freed, as if by
-     delwin.  I'm not sure if one should be able to compare windows if
-     one has been freed.  */
-  assert (_scm_is_window (x1));
-  assert (_scm_is_window (x2));
+  SCM_ASSERT (_scm_is_window (x1), x1, SCM_ARG1, "window=?");
+  SCM_ASSERT (_scm_is_window (x2), x1, SCM_ARG2, "window=?");
 
   win1 = _scm_to_window (x1);
   win2 = _scm_to_window (x2);
@@ -1093,36 +1015,23 @@ equalp_window (SCM x1, SCM x2)
     return SCM_BOOL_T;
 }
 
-static SCM
-mark_window (SCM x)
-{
-  struct gucu_window *wp;
-  wp = (struct gucu_window *) SCM_SMOB_DATA (x);
-  if (wp != (struct gucu_window *) NULL)
-    {
-      scm_gc_mark (wp->parent);
-      scm_gc_mark (wp->name);
-    }
-  return SCM_BOOL_F;
-}
-
-static size_t
+static void
 gc_free_window (SCM x)
 {
-  struct gucu_window *wp;
-  WINDOW *win;
-
-  wp = (struct gucu_window *) SCM_SMOB_DATA (x);
+  WINDOW *win = scm_foreign_object_ref (x, 0);
   /* Windows should already be null if delwin has been called on them */
-  if (wp != NULL)
+  if (win != NULL)
     {
-      wp->parent = SCM_BOOL_F;
-      wp->name = SCM_BOOL_F;
-      if (wp->panel != (PANEL *) NULL)
+      // Detach the parent window, if any.
+      scm_foreign_object_set_x (x, 2, SCM_UNPACK_POINTER (SCM_BOOL_F));
+      // The window name
+      scm_foreign_object_set_x (x, 3, SCM_UNPACK_POINTER (SCM_BOOL_F));
+      PANEL *panel = scm_foreign_object_ref (x, 1);
+      if (panel != NULL)
         {
           int retval;
-	  set_panel_userptr (wp->panel, NULL);
-          retval = del_panel (wp->panel);
+          set_panel_userptr (panel, NULL);
+          retval = del_panel (panel);
           if (retval != OK)
             {
               scm_error_scm (scm_from_locale_symbol ("ncurses"),
@@ -1131,73 +1040,25 @@ gc_free_window (SCM x)
                              scm_from_locale_string ("bad argument"),
                              SCM_BOOL_F, SCM_BOOL_F);
             }
-          wp->panel = (PANEL *) NULL;
+          scm_foreign_object_set_x (x, 1, NULL);
         }
 
-      if (wp->window != stdscr)
+      // Bad things happen if you allow the stdscr to be garbage
+      // collected.
+      if (win != stdscr)
         {
-          delwin (wp->window);
-          wp->window = (WINDOW *) NULL;
+          delwin (win);
+          scm_foreign_object_set_x (x, 0, NULL);
         }
-      SCM_SET_SMOB_DATA (x, NULL);
     }
-
-  return 0;
 }
 
 size_t
 free_window (SCM x)
 {
-  assert (SCM_SMOB_PREDICATE (window_tag, x));
-  return gc_free_window (x);
-}
-
-static int
-print_window (SCM _win, SCM port, scm_print_state * pstate UNUSED)
-{
-  struct gucu_window *wp = (struct gucu_window *) SCM_SMOB_DATA (_win);
-  char str[SIZEOF_VOID_P * 2 + 3];
-
-  scm_puts ("#<window ", port);
-
-  if (wp == (struct gucu_window *) NULL)
-    scm_puts ("(freed)", port);
-  else
-    {
-      if (wp->name != SCM_BOOL_F)
-        {
-          scm_write (wp->name, port);
-          scm_puts (" ", port);
-        }
-
-      if (wp->window == 0)
-        scm_puts ("(windowless)", port);
-      else
-	{
-	  int y, x;
-
-	  if (wp->panel != (PANEL *) NULL)
-	    scm_puts ("panel ", port);
-	  getmaxyx (wp->window, y, x);
-	  snprintf (str, sizeof (str), "%d by %d ", y, x);
-	  scm_puts (str, port);
-	  if (wp->window == stdscr)
-	    scm_puts ("stdscr ", port);
-	  if (wp->window == newscr)
-	    scm_puts ("newscr ", port);
-	  if (wp->window == curscr)
-	    scm_puts ("curscr ", port);
-
-	  if (snprintf (str, sizeof (str), "%p", (void *) wp->window) < 0)
-	    scm_puts ("???", port);
-	  else
-	    scm_puts (str, port);
-	}
-    }
-  scm_puts (">", port);
-
-  // non-zero means success
-  return 1;
+  assert (SCM_IS_A_P (x, window_fo_type));
+  gc_free_window (x);
+  return 0;
 }
 
 SCM
@@ -1206,7 +1067,8 @@ gucu_is_window_p (SCM x)
   return scm_from_bool (_scm_is_window (x));
 }
 
-
+#define u8sym(x) scm_from_utf8_symbol(x)
+#define makeFO(a,b,c) scm_make_foreign_object_type((a),(b),(c))
 void
 gucu_init_type ()
 {
@@ -1217,28 +1079,40 @@ gucu_init_type ()
 
       scm_c_define_gsubr ("mevent?", 1, 0, 0, gucu_is_mevent_p);
 
-      screen_tag = scm_make_smob_type ("screen", sizeof (SCREEN *));
-      scm_set_smob_print (screen_tag, print_screen);
-      scm_set_smob_free (screen_tag, gc_free_screen);
+      screen_fo_type = makeFO (u8sym ("screen"),
+                               scm_list_3 (u8sym ("screen"),
+                                           u8sym ("ifp"),
+                                           u8sym ("ofp")), gc_free_screen);
       scm_c_define_gsubr ("screen?", 1, 0, 0, gucu_is_screen_p);
+      scm_c_define_gsubr ("screen=?", 2, 0, 0, gucu_screen_equalp);
 
-      window_tag = scm_make_smob_type ("window", sizeof (WINDOW *));
-      scm_set_smob_mark (window_tag, mark_window);
-      scm_set_smob_free (window_tag, gc_free_window);
-      scm_set_smob_print (window_tag, print_window);
-      scm_set_smob_equalp (window_tag, equalp_window);
+      /* The window foreign object has 4 slots.
+         1. The ncurses WINDOW * structure for this window, or NULL if
+         the window has been freed.
+         2. The PANEL * if this window has been turned into a PANEL,
+         or NULL otherwise.
+         3. The SCM parent <window> of a subwin or derwin, or SCM_BOOL_F if
+         there is no parent.
+         4. The name of the window for debugging purposes, or SCM_BOOL_F
+         if there is no name. */
+      window_fo_type = makeFO (u8sym ("window"),
+                               scm_list_4 (u8sym ("window"),
+                                           u8sym ("panel"),
+                                           u8sym ("parent"),
+                                           u8sym ("name")), gc_free_window);
       scm_c_define_gsubr ("window?", 1, 0, 0, gucu_is_window_p);
+      scm_c_define_gsubr ("window=?", 2, 0, 0, gucu_window_equalp);
 
       scm_c_define_gsubr ("%scheme-char-to-c-char", 1, 0, 0,
-			  gucu_schar_to_char);
+                          gucu_schar_to_char);
       scm_c_define_gsubr ("%scheme-char-to-c-wchar", 1, 0, 0,
-			  gucu_schar_to_wchar);
+                          gucu_schar_to_wchar);
       scm_c_define_gsubr ("%scheme-char-from-c-char", 1, 0, 0,
-			  gucu_schar_from_char);
+                          gucu_schar_from_char);
       scm_c_define_gsubr ("%scheme-char-from-c-wchar", 1, 0, 0,
-			  gucu_schar_from_wchar);
+                          gucu_schar_from_wchar);
       scm_c_define_gsubr ("%xchar-from-chtype", 1, 0, 0,
-			  gucu_xchar_from_chtype);
+                          gucu_xchar_from_chtype);
       scm_c_define_gsubr ("%xchar-to-chtype", 1, 0, 0, gucu_xchar_to_chtype);
 
       first = 0;
