@@ -1,4 +1,4 @@
-;; Copyright 2009, 2010, 2016 Free Software Foundation, Inc.
+;; Copyright 2009, 2010, 2016, 2019 Free Software Foundation, Inc.
 
 ;; This file is part of Guile-Ncurses.
 
@@ -34,4 +34,4 @@
      (format #t "item1: ~s~%" item1)
      (format #t "item2: ~s~%" item2)
      (format #t "current-item: ~s~%" cur)
-     (equal? cur item2))))
+     (item=? cur item2))))
